# encoding: utf-8
# api: fpm
# title: arch tarball
# description: xzipped tarball with meta .PKGINFO and .MTREE files
# version: 0.1
# doc: https://github.com/an146/pacman/blob/master/scripts/makepkg.sh.in#L1268
#
# Arch binary packages are just .tar.xz archives with an extra .PKGINFO and
# a gzipped .MTREE (usually created by `bsdtar`) topmost in the tarball.
#
# Care must be taken with dependencies. They're mostly literal upstream
# release basenames. But there are frequently also conjoined version numbers.
# For instance python packages follow a scheme like `python2-xyz` typically.


require "backports" # gem backports
require "fpm/package"
require "fpm/package/dir"
require "fpm/version"
require "fpm/util"
require "time"
require "fileutils"
require "zlib"


# Output only
class FPM::Package::Arch < FPM::Package

  # Craft .PKGINFO and .MTREE (gzipped),
  # then generate a tarball (always .xz compressed).
  #
  def output(output_path)
    output_check(output_path)
    tarfn = File.absolute_path(output_path)
    
    ::Dir::chdir(staging_path) do
      # Convert meta
      pkginfo()
      mtree()

      # Generate tarball
      safesystem("tar", "-c", "-J", "--owner=0", "--group=0", "--xform", "s:./::", "-f", tarfn, ".")
    end
  end # def output

  # .PKGINFO
  def pkginfo()
    maparch()
    File.open(".PKGINFO", "w") do |f|
      f.write template("arch/PKGINFO.erb").result(binding)
    end
  end

  # bsdtar --format mtree
  def mtree()
    Zlib::GzipWriter.open(".MTREE") do |f|
      f.write template("arch/MTREE.erb").result(binding)
    end
  end

  # architecture  
  def maparch()
    map = {
      "all" => "any",
      "amd64" => "x86-84",
      "x86" => "i686",
    }
    if map.has_key?(architecture)
      @architecture = map[architecture]
    end
  end

  # search etc files (obsolute, use --config-files)
  def etcscan()
    return ::Dir.glob("etc/**")
  end

end # class FPM::Package::Arch
