require "backports" # gem backports
require "fpm/package"
require "fpm/util"
require "fileutils"
require "fpm/package/dir"
#require "io"

# Use a zip as a package.
#
# This provides no metadata. Both input and output are supported.
class FPM::Package::Zip < FPM::Package

  option "--shebang", "#!line", "Prefix a shebang to zip file. Make it executable.", :default => false, :hidden => true

  # Input a zipfile.
  def input(input_path)
    # use part of the filename as the package name
    self.name = File.extname(input_path)[1..-1]

    realpath = Pathname.new(input_path).realpath.to_s
    ::Dir.chdir(build_path) do
      safesystem("unzip", realpath)
    end

    # use dir to set stuff up properly, mainly so I don't have to reimplement
    # the chdir/prefix stuff special for zip.
    dir = convert(FPM::Package::Dir)
    if attributes[:chdir]
      dir.attributes[:chdir] = File.join(build_path, attributes[:chdir])
    else
      dir.attributes[:chdir] = build_path
    end

    cleanup_staging
    # Tell 'dir' to input "." and chdir/prefix will help it figure out the
    # rest.
    dir.input(".")
    @staging_path = dir.staging_path
    dir.cleanup_build
  end # def input

  # Output a tarball.
  #
  # If the output path ends predictably (like in .tar.gz) it will try to obey
  # the compression type.
  def output(output_path)
    output_check( output_path = File.absolute_path(output_path) )
    
    ::Dir.chdir(staging_path) do
      files = Find.find(".").to_a
      safesystem("zip", output_path, *files)
    end

    # Prefix a shebang
    if attributes[:zip_shebang_given?]
      File.rename(output_path, tmp = output_path + rand(36**5).to_s(36))
      File.open(tmp, "rb") do |r|
        File.open(output_path, "wb") do |w|
          w << "#!" + attributes[:zip_shebang].gsub("^\#\!\s*|\s+$", "") + "\n"
          IO.copy_stream(r, w)
        end
      end
      File.unlink(tmp)
      safesystem("zip", "-A", output_path)
      FileUtils.chmod("a+x", output_path)
    end

  end # def output

end # class FPM::Package::Zip
